@@ -5,6 +5,7 @@ from typing import Dict, Mapping, Optional, BinaryIO
 
 from kognic.base_clients.cloud_storage.download_handler import DownloadHandler
 from kognic.base_clients.cloud_storage.upload_handler import UploadHandler
+from kognic.base_clients.cloud_storage.upload_spec import UploadSpec
 
 log = logging.getLogger(__name__)
 
@@ -20,14 +21,13 @@ class FileResourceClient:
         self._upload_handler = UploadHandler(max_retry_attempts, max_retry_wait_time, timeout)
         self._download_handler = DownloadHandler(max_retry_attempts, max_retry_wait_time, timeout)
 
-    def upload_files(self, url_map: Mapping[str, str], folder: Optional[Path] = None) -> None:
+    def upload_files(self, upload_specs: Mapping[str, UploadSpec]) -> None:
         """
         Upload all files to cloud storage
 
-        :param url_map: map between filename and GCS signed URL
-        :param folder: Optional base path, will join folder and each filename in map if provided
+        :param upload_specs: Mapping from a source name to a destination and source of content for an upload
         """
-        self._upload_handler.upload_files(url_map=url_map, folder=folder)
+        self._upload_handler.upload_files(upload_specs=upload_specs)
 
     def upload_json(self, file: BinaryIO, url: str) -> None:
         """
