@@ -4,10 +4,12 @@ from pathlib import Path
 from typing import BinaryIO, Dict, Mapping, Optional
 
 import requests
-from kognic.base_clients.util import RETRYABLE_STATUS_CODES, get_content_type, get_wait_time
 from requests.exceptions import HTTPError, ConnectionError
 from requests.models import Response
 
+from kognic.base_clients.cloud_storage.upload_spec import UploadSpec, UploadableData
+from kognic.base_clients.util import RETRYABLE_STATUS_CODES, get_wait_time
+
 log = logging.getLogger(__name__)
 
 
@@ -25,14 +27,10 @@ class UploadHandler:
 
     #  Using similar retry strategy as gsutil
     #  https://cloud.google.com/storage/docs/gsutil/addlhelp/RetryHandlingStrategy
-    def _upload_file(self, upload_url: str, file: BinaryIO, headers: Dict[str, str], number_of_retries: int) -> None:
+    def _upload_file(self, upload_url: str, file: UploadableData, headers: Dict[str, str], number_of_retries: int) -> None:
         """
         Upload the file to GCS, retries if the upload fails with some specific status codes.
         """
-        if hasattr(file, "name"):
-            log.info(f"Uploading file={file.name}")
-        else:
-            log.info(f"Uploading in memory data")
         resp = requests.put(upload_url, data=file, headers=headers, timeout=self.timeout)
         try:
             resp.raise_for_status()
@@ -53,25 +51,42 @@ class UploadHandler:
         )
         time.sleep(wait_time)
 
-    def upload_files(self, url_map: Mapping[str, str], folder: Optional[Path] = None) -> None:
+    def upload_files(self, upload_specs: Mapping[str, UploadSpec]) -> None:
         """
         Upload all files to cloud storage
 
-        :param url_map: map between filename and GCS signed URL
-        :param folder: Optional base path, will join folder and each filename in map if provided
+        :param upload_specs: map between filename and details of what to upload where
         """
-        for (filename, upload_url) in url_map.items():
-            file_path = folder.joinpath(filename).expanduser() if folder else Path(filename).expanduser()
-            with file_path.open('rb') as file:
-                content_type = get_content_type(filename)
-                headers = {"Content-Type": content_type}
-                self._upload_file(upload_url, file, headers, self.max_num_retries)
+        for (resource_id, upload_spec) in upload_specs.items():
+            if not upload_spec.data and not upload_spec.callback:
+                self._upload_from_local_file(upload_spec)
+            elif upload_spec.data:
+                self._upload_from_blob(upload_spec)
+            elif upload_spec.callback:
+                self._upload_from_callback(upload_spec)
 
     def upload_file(self, file: BinaryIO, url: str) -> None:
-        """
-
-        :param file:
-        :param url:
-        """
         headers = {"Content-Type": "application/json"}
         self._upload_file(url, file, headers, self.max_num_retries)
+
+    def _upload(self, upload_spec: UploadSpec, data: UploadableData):
+        headers = {"Content-Type": upload_spec.content_type}
+        self._upload_file(upload_spec.destination, data, headers, self.max_num_retries)
+
+    def _upload_from_blob(self, upload_spec: UploadSpec):
+        log.debug(f"Blob upload for filename={upload_spec.filename}")
+        self._upload(upload_spec, upload_spec.data)
+
+    def _upload_from_local_file(self, upload_spec: UploadSpec):
+        log.debug(f"Upload from local file for filename={upload_spec.filename}")
+        file = Path(upload_spec.filename).expanduser().open('rb')
+        self._upload(upload_spec, file)
+
+    def _upload_from_callback(self, upload_spec: UploadSpec):
+        log.debug(f"Upload from callback for filename={upload_spec.filename}")
+        try:
+            data = upload_spec.callback(upload_spec.filename)
+            self._upload(upload_spec, data)
+        except Exception as e:
+            raise Exception("Failed to upload file: callback failed", e)
+        pass
