@@ -16,17 +16,6 @@ def filter_none(js: dict) -> dict:
         return js
 
 
-def get_content_type(filename: str) -> str:
-    # https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Complete_list_of_MIME_types
-    if filename.split(".")[-1] == "csv":
-        content_type = "text/csv"
-    else:
-        content_type = mimetypes.guess_type(filename)[0]
-        content_type = content_type if content_type is not None else 'application/octet-stream'
-
-    return content_type
-
-
 def get_resource_id(signed_url: str) -> str:
     url = parse_url(signed_url)
     resource_id = Url(scheme=GCS_SCHEME, path=url.path)
